local lor = require("lor.index")
local router = require("app.router")

local app = lor({debug=true}) --开启DeBug调试,正式环境请关闭

-- 模板配置, 如静态文件无法加载，参考：https://sumory.github.io/lor_site/features/issues.201.html
app:conf("view enable", true) -- 开启模板
app:conf("view engine", "tmpl") -- 模板引擎，目前仅支持lua-resty-template，此值暂时固定为"tmpl"
app:conf("view ext", "html") -- 模板文件后缀
app:conf("views", "./app/templates") -- 模板文件所在目录

-- cookie管理插件
local cookieMiddleware = require("lor.lib.middleware.cookie")
-- cookie被注入到`request`对象里: https://sumory.github.io/lor_site/features/cookie_usage.203.html
app:use(cookieMiddleware())

-- session管理插件
local sessionMiddleware = require("lor.lib.middleware.session")
-- session被注入到`request`对象里: https://sumory.github.io/lor_site/features/session_usage.204.html
app:use(sessionMiddleware({
    secret = "G3fu98Kor0rJrembv67fnhgl95FioRpQ", -- 加密用的盐
    timeout = 3600 -- session超时时间，默认为3600秒
}))

-- 请求时间插件: https://github.com/lorlabs/response-time
local reponseTime = require("lor.lib.middleware.response_time")
-- 往request对象里注入请求时间
app:use(reponseTime({
    digits = 1,
    header = 'X-Response-Time',
    suffix = true
}))

-- upload上传的插件
local uploader = require("lor.lib.middleware.uploader")
-- 设置上传文件存储路径
app:use(uploader())

-- 你可以直接设置自定义response
app:use(function(request, response, next)
    response:set_header("X-Powered-By", "Lua framework")
    next() -- 继续往下执行，参考多级处理函数：https://sumory.github.io/lor_site/router/multi_business_middleware.103.html
end)
-- 或者使用自定义插件中间件: 往request注入变量
-- 引入自定义的插件:
local testMiddleware = require("app.middleware.test")
app:use(testMiddleware())

router(app)

-- 错误处理插件，可根据需要定义多个
app:erroruse(function(error, request, response, next)
    ngx.log(ngx.ERR, error)
    if request:is_found() ~= true then -- 用于判断uri是否被路由到，如果返回false，就说明404了。
        response:status(404):send(error)
    else
        response:status(500):send(error)
    end
end)

app:run()