#!/bin/bash

#框架安装位置 
package_path="/usr/local/lor"

#获取当前脚本名称
script_name=$(basename "$0")
if [ "$script_name" != "install.sh" ]; then  
    echo "安装程序可能已被篡改，建议重新下载官方完整包"  
    exit 1  
fi 
#获取当前脚本路径
execute_path=$(cd "$(dirname "$0")" && pwd) 
#安装主程序
setup() {
    # 清理和准备  
    mkdir -p "$package_path"  
    rm -rf "$package_path"/*  
    # 复制框架文件  
    _resty="$execute_path/resty"
    _lor="$execute_path/lor"
    _bin="$execute_path/bin"
    _libs="$execute_path/libs"
    for dir in $_resty $_lor $_bin $_libs; do  
        if [ ! -d "$dir" ]; then  
            echo "框架安装结束：安装包不完整请重新下载！"  
            return 1  
        fi  
        echo "复制$dir到${package_path}/"  
        cp -r "$dir" "${package_path}/"  
    done  
    cp "$execute_path/install.sh" "$package_path/"
    echo '================================================'  
    echo "框架安装成功！"  
} 

#检查安装环境
check() {
    echo "检查安装环境中..."  
    # 是否为Nginx框架 
    nginx_version=$(nginx -v 2>&1)  
    if [ $? -ne 0 ]; then  
        echo "框架安装结束：仅支持Nginx框架！"  
        return 1  
    fi  
    # 是否有OpenResty模块
    if [[ ! "$nginx_version" == *"openresty"* ]]; then  
        echo "框架安装结束：Nginx未安装OpenResty模块！"  
        return 1  
    fi
    setup
}

# 安装
install() {
    echo '================================================'  
    echo "开始安装框架..."
    # 框架是否已安装
    if [ -d "${package_path}/" ]; then  
        read -p "框架可能已安装，确定继续清理旧版并重装？(y/n): " confirmation  
        if [ "$confirmation" == "y" ] || [ "$confirmation" == "Y" ] || [ "$confirmation" == "yes" ] || [ "$confirmation" == "Yes" ] || [ "$confirmation" == "YES" ]; then  
            echo "正在清理旧版数据..." 
            rm -rf "${package_path}"  
            echo "正在准备重新安装中..."  
            check
        else  
            echo "框架安装结束：取消安装！"  
        fi  
        exit 1
    fi
    check
}

# 卸载
uninstall() {  
	echo '================================================'  
	echo "准备卸载框架..."
    if [ ! -d "${package_path}/" ]; then
        echo "卸载失败：框架不存在,可能未安装！"
        exit 1
    fi
    echo "请稍等,等待初始化..."
    read -p "是否确定卸载框架清理数据？(y/n): " confirmation  
    if [ "$confirmation" == "y" ] || [ "$confirmation" == "Y" ] || [ "$confirmation" == "yes" ] || [ "$confirmation" == "Yes" ] || [ "$confirmation" == "YES" ]; then  
        echo "正在卸载中..." 
        rm -rf "${package_path}" 
        echo "正在清理数据..." 
        echo '================================================'  
        echo "框架卸载完成！"  
    else  
        echo "框架卸载失败：取消卸载！"  
    fi  
}

#操作判断
if [ "${1}" == '-uninstall' ] || [ "${1}" == 'uninstall' ]; then
	uninstall #运行卸载程序
else
	install #运行安装程序
fi