local http = require("resty.http")
local cjson = require("cjson")
local iconv = require("resty.iconv")

local _M = {}
function _M.request(url, method, body, headers, cookies, user_agent, callback)

    local httpc = http.new()
    
    method = method or "GET"
    headers = headers or {}
    body = body or {}
 
    if cookies then
        headers["Cookie"] = cookies
    end
    if user_agent then
        headers["User-Agent"] = user_agent
    end

    local is_json, result = pcall(cjson.decode, body)
    if type(body) == "table" or is_json then
        headers["Content-Type"] = "application/json"
        body = cjson.encode(body)
    else
        headers["Content-Type"] = "application/x-www-form-urlencoded"
    end
    local response, error = httpc:request_uri(url, {
        method = method,
        headers = headers,
        body = body
    })

    local headers = response.headers
    local status = response.status
    local body = response.body or ""
    local _error
    if body and error then
        error = body .. " " .. error
    elseif body then
        error = body
    else
        error = error or "Unknown error"
    end
    if status == 200 then
        local content_type = headers["Content-Type"] or ""
        local charset = string.match(content_type, "charset=([%w-]+)")
        local ic = iconv.new("utf-8", charset)
        if charset:lower() ~= "utf-8" then
            local utf8_body, iconv = ic:convert(body)
            if utf8_body then
                body = utf8_body
            end
        end
    end
    if type(callback) == "function" then
        callback(status, body, error)
    end
end

function _M.get(url, cookies, user_agent, headers, callback)
    _M.request(url, "GET", "", headers, cookies, user_agent, callback)
end

function _M.post(url, body, cookies, user_agent, headers, callback)
    _M.request(url, "POST", body, headers, cookies, user_agent, callback)
end

return _M