local upload = require("resty.upload")
local resty_md5 = require("resty.md5")
local resty_string = require("resty.string")

local sfind = string.find
local match = string.match
local ngx_var = ngx.var

local function get_filename(header_value)
   local file_name
    for i, ele in ipairs(header_value) do
        file_name = string.match(ele, 'filename="(.*)"')
        if file_name and file_name ~= '' then
            return file_name
        end
    end
    return nil
end

local function get_extension(file_name)
	return file_name:match(".+%.(%w+)$") or false
end

local function set_filename(temp_path, file_path)
    local success = os.execute("mv '" .. temp_path .. "' '" .. file_path .. "'")
    if success or success == 0 then  
        return true
    else  
        return false
    end
end

local function _multipart_formdata(config)

	local form, err = upload:new(4096)
	if not form then
		ngx.log(ngx.ERR, "failed to new upload: ", err)
		ngx.exit(500)
	end
	form:set_timeout(60000)
	
	local calculate_md5 = resty_md5:new()
	local success, msg = false, ""
	local filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
	
	while true do
		local type, res, error = form:read()
		if not type then
		    success = false
			msg = "failed to read: " .. error
			ngx.log(ngx.ERR, msg)
			return success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
		end
		
		if type == "header" then
		    local key = res[1]
            local value = res[2]
		    if key == "Content-Disposition" then
		        filename = get_filename(res)
			    if filename then
			    	file_type = get_extension(filename)
			    	path = "/tmp/"
			    	temp_path = path .. filename
			    	file = io.open(temp_path, "w+")
			    	if not file then
			    	    success = false
			    	    msg = "open file error"
			    		ngx.log(ngx.ERR, msg)
			    		return success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
			    	end
			    else
			        success = false
			    	msg = "failed to get filename."
			    	ngx.log(ngx.ERR, msg)
			    	return success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
			    end
		    end
		elseif type == "body" then
			if file then
			    file:write(res)
			    calculate_md5:update(res)
				success = true
			else
				success = false
				msg = "upload file error, path:" .. temp_path
				ngx.log(ngx.ERR, msg)
				return success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
			end
		elseif type == "part_end" then
            file:close()
            file = nil
            file_md5 = resty_string.to_hex(calculate_md5:final())
            file_name = file_md5
            if file_type then
                file_name = file_md5 .. "." .. file_type
            end
            file_path = path .. file_name
            calculate_md5:reset()
            local rename_success = set_filename(temp_path, file_path)
            if not rename_success then
                success = false
                msg = "rename failed."
                ngx.log(ngx.ERR, msg)
                return success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
			end
		elseif type == "eof" then
			break
	    end
	end

	return success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress
end

local function uploader()
	return function(request, response, next)

		if ngx_var.request_method == "POST" then
			local get_headers = ngx.req.get_headers()
			local header = get_headers['Content-Type']
			request.file = request.file or {}
			if header then
				local is_multipart = sfind(header, "multipart")
				--local is_octet_stream = sfind(header, "octet-stream")
				if (is_multipart and is_multipart>0) then
					local success, msg, filename, file_type, path, temp_path, file, file_md5, file_name, file_path, typ, ress = _multipart_formdata()
					request.file.success = success
					request.file.error = msg
					request.file.type = file_type
					request.file.path = file_path
					request.file.name = file_name
					request.file.md5 = file_md5
					next()
				else
				    request.file.success = false
					request.file.error = "Only MIME type of 'multipart/form-data' is supported."
					request.file.type = ""
					request.file.path = ""
					request.file.name = ""
					request.file.md5 = ""
					next()
				end
			else
				next()
			end
		else
			next()
		end
	end
end

return uploader