#!/bin/bash

project_name="" #项目名称
project_port="" #项目端口号
#获取当前格式化时间
current_time=$(date '+%Y/%m/%d %H:%M:%S')
#获取当前脚本名称
script_name=$(basename "$0") 
#获取当前脚本路径
execute_path=$(cd "$(dirname "$0")" && pwd) 
#框架安装脚本路径  
install_script="./framework/install.sh"  
#框架默认安装路径 
package_path="/usr/local/lor"
#宝塔网站数据库位置
bt_site_db="/www/server/panel/data/db/site.db"

#启动项目
run() {  
    #运行先判断框架是否成功安装
    if [ ! -d "$package_path/" ]; then
        echo "$current_time 框架已损坏或未安装, 请重新安装框架！"
        exit 1
    fi
    #数据库是否存在且可读, 需要权限才可以读取
    if [ ! -f "$bt_site_db" ]; then  
        echo "$current_time 请使用root用户启动"  
        exit 1
    fi  
    #缺少运行必要组件 
    if ! command -v sqlite3 &> /dev/null; then  
        echo "当前系统缺少SQLite3,请先安装再运行"  
        exit 1  
    fi 
    #获取宝塔项目详情
    project_config=$(sqlite3 "$bt_site_db" "SELECT project_config FROM sites WHERE path LIKE '%$execute_path%';")  
    if [ -z "$project_config" ]; then  
        #未添加网站直接运行项目
        echo "$current_time 请勿直接运行本脚本！"
        exit 1
    fi
    project_name=$(echo "$project_config" | sed -n 's/.*"project_name": "\([^"]*\)".*/\1/p')
    project_port=$(echo "$project_config" | sed -n 's/.*"port": \([0-9]*\).*/\1/p') 
    #检查项目名和端口号  
    if [ -z "$project_name" ] || [ -z "$project_port" ]; then  
        echo "$current_time 项目名称为空或端口无效,请检查项目名和端口"  
        exit 1
    fi
    #检查是否已经在运行
    if ps -eo pid,cmd --no-headers | grep -q "[n]ginx.*$execute_path/"; then  
        echo "$current_time 该项目已经在运行,暂不支持多开,请停止或结束进程"  
        exit 1
    fi 
    #检查端口是否被占用
    if sudo ss -ltnp | grep -q ":$project_port"; then 
        echo "$current_time 端口$project_port已被使用,请更换端口或结束该进程"  
        exit 1
    fi
    #创建运行必要文件
    mkdir -p conf logs tmp
    #生成项目配置文件
    #template_root = '\$template_root'
    sed -i -e "s/listen [0-9]\+/listen ${project_port}/g" -e "s|/www/wwwlogs/other/\([^/]*\).log|/www/wwwlogs/other/${project_name}.log|g" "conf/nginx.conf"
    if [ $? -ne 0 ]; then 
        echo "$current_time 生成配置文件nginx.conf失败,请重试" 
        exit 1  
    fi 
    #加载项目配置启动
    nginx_results=$(nginx -p "$(pwd)"/ -c conf/nginx.conf 2>&1) 
    if [ $? -ne 0 ]; then  
        echo "$current_time 启动失败,因为$nginx_results" #项目无法正常启动
        exit 1
    fi
    #获取项目运行进程
    nginx_pid=$(ps -eo pid,cmd --no-headers | grep "$execute_path/" | head -n 1 | awk '{print $1}')
    if [ -z "$nginx_pid" ]; then  
        echo "$current_time 无法获取$execute_path/项目进程PID,请重试"
        exit 1
    fi
    #强制覆写宝塔PID
    project_pid="/var/tmp/other_project/$project_name.pid"
    echo "$nginx_pid" > "$project_pid"
    echo "$current_time $project_name在端口$project_port上启动成功!"
    #没用的文件夹删掉
    rm -rf logs
}

#安装框架
install() {
    #判断框架安装脚本存不存在
    if [ -f "$install_script" ]; then  
        bash "$install_script" 
    else  
        echo "$current_time 无法安装框架: install.sh不存在! 请手动下载并安装"  
        exit 1
    fi
}

#操作判断
if [ "${1}" == '-install' ] || [ "${1}" == 'install' ]; then
    install #尝试安装框架
else
	run #运行启动程序
fi